/* -*-c-*-
 *
 * Implementation of double- and triple-DES
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

#ifndef CATACOMB_DES3_H
#define CATACOMB_DES3_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Notes on double- and triple-DES -----------------------------------*
 *
 * The normal recommendation for using DES now is `triple DES'.
 * Conventionally, this involves an encrypt-decrypt-encrypt sequence,
 * although whether the first and last operations use the same key is
 * unfortunately not agreed upon.  This interface handles both cases (and the
 * single-key one too, although the simple @des@ calls are quicker for that).
 */

/*----- Header files ------------------------------------------------------*/

#include <stddef.h>

#include <mLib/bits.h>

#ifndef CATACOMB_DES_H
#  include "des.h"
#endif

/*----- Magical numbers ---------------------------------------------------*/

#define DES3_BLKSZ 8
#define DES3_KEYSZ 21
#define DES3_CLASS (N, B, 64)

extern const octet des3_keysz[];

/*----- Data structures ---------------------------------------------------*/

typedef struct des3_ctx {
  des_ctx a, b, c;
} des3_ctx;

/*----- Functions provided ------------------------------------------------*/

/* --- @des3_init@ --- *
 *
 * Arguments:	@des3_ctx *k@ = pointer to key block
 *		@const void *buf@ = pointer to key buffer
 *		@size_t sz@ = size of key material
 *
 * Returns:	---
 *
 * Use:		Initializes a DES key buffer.  The key buffer may have length
 *		7, 8, 14, 16, 21, or 24.  These correspond to one, two or
 *		three DES keys, either packed or unpacked (i.e., still
 *		containing parity bits).
 */

extern void des3_init(des3_ctx */*k*/, const void */*buf*/, size_t /*sz*/);

/* --- @des3_eblk@, @des_dblk@ --- *
 *
 * Arguments:	@const des3_ctx *k@ = pointer to key block
 *		@const uint32 s[2]@ = pointer to source block
 *		@uint32 d[2]@ = pointer to destination block
 *
 * Returns:	---
 *
 * Use:		Low-level block encryption and decryption.
 */

extern void des3_eblk(const des3_ctx */*k*/,
		      const uint32 */*s*/, uint32 */*d*/);
extern void des3_dblk(const des3_ctx */*k*/,
		      const uint32 */*s*/, uint32 */*d*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
