/* -*-c-*-
 *
 * Finding primitive elements
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

#ifndef CATACOMB_PRIM_H
#define CATACOMB_PRIM_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <stddef.h>

#ifndef CATACOMB_MP_H
#  include "mp.h"
#endif

#ifndef CATACOMB_MPMONT_H
#  include "mpmont.h"
#endif

#ifndef CATACOMB_PGEN_H
#  include "pgen.h"
#endif

/*----- Data structures ---------------------------------------------------*/

/* --- @prim_ctx@ --- *
 *
 * All fields must be configured by the client.  Set @n@ to zero to discover
 * generators of the subgroup of order %$m / f$%.
 *
 * Let %$p = \prod q_i + 1$% be a prime number.  In order to find an element
 * %$g$% with order %$o$%, we choose elements %$h_j$% from %$\gf{p}^*$%,
 * compute $%g_j = h_j^{p/o}$%, rejecting %$h_j$% where %$g_j = 1$%, and then
 * for each proper prime factor %$q_i$% of %$p/o$% we check that
 * %$g^{f_i} \ne 1$%, where the %$f_i$% are cofactors of the %$q_i$%
 * (%$f_i q_i = p/o$%).
 */

typedef struct prim_ctx {
  mpmont mm;				/* Montgomery context for modulus */
  mp *exp;				/* Exponent (%$p/o$%; may be zero) */
  size_t n;				/* Number of cofactors */
  mp **f;				/* Array of cofactors */
} prim_ctx;

/*----- Functions provided ------------------------------------------------*/

/* --- @prim_test@ --- */

extern pgen_proc prim_test;

/* --- @prim_step@ --- */

extern pgen_proc prim_step;

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
