/* -*-c-*-
 *
 * Quick-and-dirty parser
 *
 * (c) 2004 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

#ifndef CATACOMB_QDPARSE_H
#define CATACOMB_QDPARSE_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#ifndef CATACOMB_MP_H
#  include "mp.h"
#endif

/*----- Data structures ---------------------------------------------------*/

typedef struct qd_parse {
  const char *p;			/* Where we are right now */
  const char *e;			/* Error string (output) */
} qd_parse;

/*----- Functions provided ------------------------------------------------*/

/* --- @qd_skipspc@ --- *
 *
 * Arguments:	@qd_parse *qd@ = context
 *
 * Returns:	---
 *
 * Use:		Skips spaces in the string.  No errors.
 */

extern void qd_skipspc(qd_parse */*qd*/);

/* --- @qd_delim@ --- *
 *
 * Arguments:	@qd_parse *qd@ = context
 *		@int ch@ = character to compare with
 *
 * Returns:	Nonzero if it was, zero if it wasn't.
 *
 * Use:		Checks the next (non-whitespace) character is what we
 *		expect.  If it is, the character is eaten; otherwise it's no
 *		big deal.
 */

extern int qd_delim(qd_parse */*qd*/, int /*ch*/);

/* --- @qd_enum@ --- *
 *
 * Arguments:	@qd_parse *qd@ = context
 *		@const char *e@ = list of enum strings, space separated
 *
 * Returns:	Index of the string matched, or @-1@.
 *
 * Use:		Matches a keyword.
 */

extern int qd_enum(qd_parse */*qd*/, const char */*e*/);

/* --- @qd_getmp@ --- *
 *
 * Arguments:	@qd_parse *qd@ = context
 *
 * Returns:	The integer extracted, or null.
 *
 * Use:		Parses a multiprecision integer from a string.
 */

extern mp *qd_getmp(qd_parse */*qd*/);

/* --- @qd_eofp@ --- *
 *
 * Arguments:	@qd_parse *qd@ = context
 *
 * Returns:	Nonzero if at EOF, zero otherwise.
 */

extern int qd_eofp(qd_parse */*qd*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
